@@ -1,4 +1,3 @@
-
 /*
  * Copyright 1986 by Larry Campbell, 73 Concord Street, Maynard MA 01754 USA
  * (maynard!campbell).  You may freely copy, use, and distribute this software
@@ -23,13 +22,14 @@ extern int tty;
 
 /* ARGSUSED */
 
-c_reset(argc, argv)
+void c_reset(argc, argv)
 char *argv[];
 {
-    int hcode, hletter, n;
+    int hletter;
     int oldcode;
-    char buf[20];
     extern int x10_housecode;
+    extern int char2hc();
+    extern int c_setclock(), usage();
 
     if (argc > 3)
 	usage(E_WNA);
