@@ -24,6 +24,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <stdlib.h>
 #ifdef SYSV
 #include <string.h>
 #else
@@ -39,18 +40,27 @@
 #include <sys/types.h>
 #endif
 
+/* I'm not sure why I had these (time and localtime) in here.  Was there
+a system where time.h wasn't enough?
+
 extern long time();
 extern struct tm *localtime();
+*/
 extern int tty;
 extern int sptty;
 extern char x10_tty[];
+extern int usage();
+extern int start_relay(),setup_sp_tty(), xread(), unit2int();
+extern void error(), quit();
 
 int x10_housecode;
 int verbose;
 int i_am_relay = 0;
 
-void sigtimer();
+void sigtimer(), err();
 char hc2char();
+void init();
+int check4poll();
 
 char
  syncmsg[SYNCN], flag;
@@ -100,14 +110,17 @@ unsigned char map2cm11[]=
     13, 5, 3, 11, 15, 7, 1, 9, 14, 6, 4, 12, 16, 8, 2, 10
 };
 
-int timeout = TIMEOUT, Iloaded, Iminutes, Iseconds, Ihours, Idays, Ijday;
+int timeout = TIMEOUT, Irev, Iminutes, Iseconds, Ihours, Idays, Ijday;
 int Istatdim, Istatmon, Iaddmon;
+int newformat;  /* for the output format */
 unsigned char Ihcode;
 
 extern int
  c_data(), c_date(), c_delete(), c_diagnostic(), c_dump(), c_fdump(),
  c_finfo(), c_fload(), c_info(), c_monitor(), c_reset(), c_schedule(),
- c_setclock(), c_turn(), c_dbs(), c_stop(), c_erase();
+ c_setclock(), c_turn(), c_dbs(), c_stop(), c_erase(), c_preset();
+
+extern int c_load_macro(), c_qerase();
 
 int c_help(), c_version();
 
@@ -115,33 +128,40 @@ struct cmdentry {
     char *cmd_name;
     int (*cmd_routine) ();
 } cmdtab[] = {
-    "date", c_date,
-    "info", c_info,
-    "monitor", c_monitor,
-    "reset", c_reset,
-    "setclock", c_setclock,
-    "status", c_turn,
-    "stop", c_stop,
-    "turn", c_turn,
-    "erase", c_erase,
-    "version", c_version,
-    "help", c_help,
-    "", NULL
+    {"date", c_date} ,
+    {"erase", c_erase},
+    {"help", c_help},
+    {"info", c_info},
+    {"macro", c_load_macro},
+    {"monitor", c_monitor},
+    {"preset", c_preset},
+    {"reset", c_reset},
+    {"setclock", c_setclock},
+    {"status", c_turn},
+    {"stop", c_stop},
+    {"turn", c_turn},
+    {"upload", c_load_macro},
+    {"version", c_version},
+    {"qerase", c_qerase},  /* hidden feature */
+    {"", NULL }
 };
 
 char *argptr;
 
-main(argc, argv)
+int main(argc, argv)
 int argc;
 char *argv[];
 {
-    register i;
+    register int i;
     int (*rtn) ();
     struct cmdentry *c;
     struct stat statb;
-    char RCSID[]= "@(#) $Id: x10.c,v 1.13 1998/04/28 04:53:25 dbs Exp $\n";
+    char RCSID[]= "@(#) $Id: x10.c,v 1.26 1999/12/26 20:37:17 dbs Exp $\n";
+    void read_config();
 
 
+    display(RCSID);
+
     rtn = NULL;
     verbose = 1;		/* for the error message */
     if( argc > 1)
@@ -229,10 +249,10 @@ char *argv[];
  */
 
 
-dowX2U(b)
+int dowX2U(b)
 register char b;
 {
-    register n;
+    register int n;
 
     for (n = 1; (!(b & 1)) && n < 8; n++, b = b >> 1)
     	;
@@ -243,20 +263,17 @@ register char b;
     return (n);
 }
 
-dowU2X(d)
-register d;
+int dowU2X(d)
+int d;
 {
     if (d == 0)
 	d = 7;
     return (1 << (d - 1));
 }
 
-init(cmd_name)
+void init(cmd_name)
 char * cmd_name;
 {
-    int n;
-    unsigned int batlife;
-    unsigned char buf[30];
     char *bits2day();
     int get_status();
 
@@ -279,7 +296,7 @@ char * cmd_name;
  * chksum is used to check the CM11A's acknowledgment to the
  * data we send it.  It's a simple addition of all bytes.
  */
-chksum(buf, size)
+int chksum(buf, size)
 unsigned char *buf;
 int size;
 {
@@ -293,7 +310,7 @@ int size;
 char hc2char(code)
 unsigned code;
 {
-    register i;
+    register int i;
 
     for (i = 0; i < 16; i++)
 	if (housetab[i].h_code == code)
@@ -301,15 +318,16 @@ unsigned code;
     return ('?');
 }
 
-char2hc(hletter)
+int char2hc(hletter)
 int hletter;
 {
     if (isupper(hletter))
 	hletter = tolower(hletter);
-    if ('a' <= hletter && hletter <= 'p')
-        return housetab[hletter - 'a'].h_code;
-    else
+    if (! ('a' <= hletter && hletter <= 'p') )
 	error("invalid house code");
+
+    return (housetab[hletter - 'a'].h_code);
+
 }
 
 /*
@@ -317,7 +335,7 @@ int hletter;
  * (big-endian) of units specified.  '*' means "all units".
  */
 
-getunits(p)
+unsigned int  getunits(p)
 register char *p;
 {
 
@@ -348,28 +366,42 @@ register char *p;
 	}
 
 	if (*p)
-	    if (*p == '-') {
+	{
+	    if (*p == '-')
+	    {
 		lastunit = unit;
 		range = 1;
 		p++;
-	    } else if (*p == ',') {
-		lastunit = 0;
-		range=0;
-		p++;
-	    } else {
-		error("bad unit separator, use comma or dash please");
 	    }
+	    else
+	    {
+		if (*p == ',')
+		{
+		    lastunit = 0;
+		    range=0;
+		    p++;
+		}
+		else
+		{
+		    error("bad unit separator, use comma or dash please");
+		}
+	    }
+	}
     }
     return (bitmsk);
 }
 
-dimstate(p, level)
+int dimstate(p, level)
 register char *p, *level;
 {
     unsigned levelnum;
     unsigned dim;
 
-    dim = 0;
+    dim = -1;
+    if (strcmp(p, "alloff") == 0)
+	return (0);
+    if (strcmp(p, "lightson") == 0)
+	return (1);
     if (strcmp(p, "on") == 0)
 	return (2);
     if (strcmp(p, "off") == 0)
@@ -378,7 +410,9 @@ register char *p, *level;
 	dim = 4;
     if (strcmp(p, "bright") == 0)
 	dim = 5;
-    if (dim == 0)
+    if (strcmp(p, "lightsoff") == 0)
+	return(6);
+    if (dim == -1)
 	error("bad state keyword");
     if (sscanf(level, "%d", &levelnum) == 0)
 	error("dim value must be numeric");
@@ -390,23 +424,22 @@ register char *p, *level;
 
 struct nstruct dtab[] =
 {
-    "monday", 0x01,
-    "tuesday", 0x02,
-    "wednesday", 0x04,
-    "thursday", 0x08,
-    "friday", 0x10,
-    "saturday", 0x20,
-    "sunday", 0x40,
-    "everyday", 0x7f,
-    "weekdays", 0x1f,
-    "weekends", 0x60,
-    "", 0x00
+    {"monday", 0x01},
+    {"tuesday", 0x02},
+    {"wednesday", 0x04},
+    {"thursday", 0x08},
+    {"friday", 0x10},
+    {"saturday", 0x20},
+    {"sunday", 0x40},
+    {"everyday", 0x7f},
+    {"weekdays", 0x1f},
+    {"weekends", 0x60},
+    {"", 0x00 }
 };
 
-day2bits(p)
+int day2bits(p)
 char *p;
 {
-    char c, buf[6];
     int n, mask, length;
 
     if (strcasecmp(p,"Today") == 0)
@@ -425,7 +458,7 @@ char *p;
     return (mask);
 }
 
-#define MODULES 25
+#define MODULES 255
 #define NAME_LEN 20
 #define UNIT_LEN 50
 
@@ -436,7 +469,7 @@ struct x10_mod {
 } x10_modules[MODULES];
 
 
-read_config()
+void read_config()
 {
 
 	char line[256];
@@ -445,6 +478,7 @@ read_config()
 	char unit[UNIT_LEN];
 	char *configfile;
 	char * home;
+	struct stat file_buf;
 	static FILE *f = NULL;
 	int i = 0;
 	int n;
@@ -464,17 +498,21 @@ read_config()
 		configfile = line;
 	}
 
+	if( stat(configfile,&file_buf) != 0 )
+	        configfile = "/etc/x10.config";
+
 	if( verbose )
 		fprintf(stderr,
 		    "Using the config file %s\n", configfile);
 	f = fopen(configfile,"r");
 	if (!f) {
-		err("couldn't open configfile %s: is $X10CONFIG set?",
+		err("couldn't open configfile %s: is $X10CONFIG set?\n See the  x10config man page.",
 			configfile);
 		exit(1);
 	}
 
 	clearerr(f);
+
 	for (;;)
 	{
 		line [0] = '\0';
@@ -482,11 +520,17 @@ read_config()
 		fgets(line, 255, f);
 
 		if ((line[0] == '\0') || (line[0] == 0x0a) )
+		{
 		    if( feof(f)  != 0 )		/* end of file */
+		    {
 			break;
+		    }
 		    else
+		    {
 			continue;		/* blank line */
+		    }
 
+		}
 		if (i >= MODULES)
 		{
 		    err("out of table space for config", NULL);
@@ -544,7 +588,14 @@ read_config()
 			break;
 
 		    case 1:
-			err("warning: short field in line in config:\n	%s",line);
+			if (strcmp(alias, "NEWFORMAT") == 0)
+			{
+				/* another special case */
+				newformat = 1;
+				continue;
+			}
+			else
+			    err("warning: short field in line in config:\n	%s",line);
 			break;
 		    default:
 			err("warning: too many fields in line in config:\n	%s",line);
@@ -552,6 +603,9 @@ read_config()
 		}
 
 	}
+	x10_modules[i].hc = hc2char(x10_housecode);
+	strcpy(x10_modules[i].un,"*");
+	strcpy(x10_modules[i].name, "all");
 
 	fclose (f);
 
@@ -603,14 +657,13 @@ char *unit;
 }
 
 
-parse_unit(name,hcp,unitp)
+void parse_unit(name,hcp,unitp)
 char *name;
 int *hcp;
 char **unitp;
 {
     struct x10_mod *x;
     int hletter;
-    static int unitnum[5];
 
     if (isalpha(name[0]) && (isdigit(name[1]) || name[1] == '*')) {
 	hletter = name[0];
@@ -631,7 +684,7 @@ char **unitp;
 }
 
 
-err(m,s)
+void err(m,s)
 char *m, *s;
 {
 	fprintf(stderr, "heyu: ");
@@ -675,19 +728,20 @@ dump_config()
  *
  *  If the showdata flag is set, we print.  Otherwise we just eat the output.
  */
-check4poll(showdata, timeout)
+int check4poll(showdata, timeout)
 int showdata;
 int timeout;
 {
     int n, i;
     int to_read;
     char hc;
+    char *nl;
     int unit;
     int macro_report;
     char *func;
     int funcbits;
     static int wasflag = 0;
-    unsigned char buf[15];
+    unsigned char buf[128];
     char statstr[80];
     extern char *funcmap[];
     extern char *b2s();
@@ -704,8 +758,16 @@ int timeout;
 	timestore = 0;
 	time(&timestore);
 	tp = localtime(&timestore);
-	sprintf(statstr, "%s at %02d:%02d:%02d ",
-	wdays[tp->tm_wday], tp->tm_hour, tp->tm_min, tp->tm_sec);
+	if( newformat == 1 )
+	{
+	    sprintf(statstr, "%d/%d at %02d:%02d:%02d ",
+	    tp->tm_mon + 1, tp->tm_mday, tp->tm_hour, tp->tm_min, tp->tm_sec);
+	}
+	else
+	{
+	    sprintf(statstr, "%s at %02d:%02d:%02d ",
+	    wdays[tp->tm_wday], tp->tm_hour, tp->tm_min, tp->tm_sec);
+	}
 
     }
     if( timeout == 0 )		/* only do a read if there is data */
@@ -751,7 +813,7 @@ int timeout;
 	{
 	    n = buf[0];
 	    for(i = 0; i < wasflag;i++)
-		buf[i] == 0xff;
+		buf[i] = 0xff;
 	    buf[i] = n;
 	    wasflag = 0;
 	}
@@ -802,10 +864,15 @@ int timeout;
 
 		if( macro_report == 1 ) /* CM11A is reporting a macro execution.*/
 		{
-		    if( showdata )
-			fprintf( stderr, "%s executed macro   : macro executed at eeprom address %d\n",
+		    if( showdata ) 
+		    {
+			if( newformat == 1 )
+			    fprintf( stderr, "%s Triggered macro  : macro executed at eeprom address %d\n",
 				statstr, ((buf[0] & 0x07) << 8) + buf[1] );
-
+			else
+			    fprintf( stderr, "%s executed macro   : macro executed at eeprom address %d\n",
+				statstr, ((buf[0] & 0x07) << 8) + buf[1] );
+		    }
 		}
 		else
 		{
@@ -831,7 +898,6 @@ int timeout;
 			}
 			else if( (buf[0] & (0x01 << (i-1))) != 0)   /* a function */
 			{
-			    char *nl;
 			    hc = hc2char( ((buf[i] & 0xF0) >> 4) );
 			    funcbits = buf[i] & 0x0F;
 			    func = funcmap[ buf[i] & 0x0F];
@@ -842,7 +908,7 @@ int timeout;
 				nl = "\n";
 				
 			    if( showdata )
-				fprintf( stderr, "%s function  %06s : housecode %c%s",
+				fprintf( stderr, "%s function  %6s : housecode %c%s",
 					statstr, func, hc, nl);
 			}
 			else		/* an address */
@@ -876,6 +942,11 @@ int timeout;
 		n = xread(sptty, buf, to_read, timeout);
 		if(showdata)
 		{
+		    /* format changed with version 1.28c */
+		    if( newformat == 1)
+			fprintf(stderr, "%s Timer executed   : macro at eeprom address %d\n",
+				statstr, (((buf[0] & 0x07)<<8) + buf[1])); 
+		    else
 		    fprintf(stderr, "%s executed macro at EEPROM address %d\n",
 				statstr, (((buf[0] & 0x07)<<8) + buf[1])); 
 		    /*   TEST CODE
@@ -916,14 +987,46 @@ int b;
        case 64:
 	   return("Sat");
     }
+    return("Error");
 }
 
-c_version()
+int c_version()
 {
-	printf( "Version:%4s\n", VERSION );
+    printf( "Version:%4s\n", VERSION );
+    return(0);
 }
 
-c_help()
+int c_help()
 {
     usage(NULL);
+    return(0);
+}
+
+void display(RCCS)
+char *RCCS;
+{
+
+    /* This is not used much so far */
+    if( verbose > 1)
+        printf( "%s\n", RCCS);
+}
+
+/* take the simple bitmap produced by getunits() and turn it into the
+ * cm11 style, where bit 0 represents unit 13, bit 1 represents unit 5, etc
+ */
+unsigned int cm11bitmap(bits)
+unsigned int bits;
+{
+    unsigned int cm11bits;
+    int x;
+
+    cm11bits=0;
+    for(x=0; x < 16; x++)
+    {
+        if( ( bits & (1 << x ) ) != 0 )
+            cm11bits |= (1 << (cm11map[x]) );
+    }
+
+
+return(cm11bits);
 }
